/*
===============================================================================

  FILE:  svreaddereferenced.h
  
  CONTENTS:
  
    A filter that provides dereferenced access to a streaming mesh. Whenever
    a tetrahedron is read there are explicit pointers to the property fields
    of the four vertices available (i.e. read as 'polyhedra soup'). 
  
  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2007 martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    08 October 2007 -- created after bringing my new Gevalia machine to work
  
===============================================================================
*/
#ifndef SVREAD_DEREFERENCED_H
#define SVREAD_DEREFERENCED_H

#include "svreader.h"

class SVreadDereferenced : public SVreader
{
public:

  // additional cell variables

  float** c_v_prop_f;

  // SVreader interface function implementations

  virtual SVevent read_element();
  virtual SVevent read_event();

  virtual void close(bool close_file=true);

  // SVreadDereferenced functions

  bool open(SVreader* svreader);

  SVreadDereferenced();
  virtual ~SVreadDereferenced();

private:
  SVreader* svreader;

  int have_new, next_new;
  int new_vertices[8];
  int have_cell;
  int have_finalized, next_finalized;
  int finalized_vertices[8];

  int read_input();
  int read_cell();
};

#endif
