/*
===============================================================================

  FILE:  SVreader_geo.h

  CONTENTS:

    Reads a tranfinite geometry from GMSH .geo file, and meshes this geometry.
    NOTE: for a smaller memory footprint, transfinite lines defibition should
    come before the blocks are defined. That way, the mesher can choose the
    surface with smallest transfinite area as advancing front.

    Restrictions: 
      - No Splines/BSplines
      - only manifold
      - no constant definition
      - only type 1 progression (uniform subdivision)
      - only convex blocks (can actually be non-convex to some extent)
      - No translation/extrusion/.... Note that these operations can still be used by opening and saving the file with such operations with gmsh.

  PROGRAMMERS:

    clement.courbet@ecp.fr

  COPYRIGHT:

    copyright (C) 2009 clement courbet

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

  CHANGE HISTORY:

    23 November 2009 -- Created

===============================================================================
*/
#ifndef SVREADER_GEO_H
#define SVREADER_GEO_H

#include "svreader.h"

#include <stdio.h>

class SVreader_geo : public SVreader
{
  public:
    // SVreader interface function implementations

    SVevent read_element();
    SVevent read_event();

    void close(bool close_file=true);

    // SVreader_geo functions

    bool open(FILE* fp);

    SVreader_geo();
    ~SVreader_geo();

  private:
    FILE* file;
    int have_finalized, next_finalized;
    int* finalized_vertices;

    int currentBlock;
    I32 *next_idx;
    bool *next_final;

    int indexBase;
    bool read_geometry();
};

#endif
