/*
===============================================================================

  FILE:  SVreader_hzip.h

  CONTENTS:

    Reads a hexahedral volume mesh from a HZIP format.
    While the writer for this format is streaming, the reader is not, because the HZIP file format does not support finalization information.
    see writer for details.

  PROGRAMMERS:

    clement.courbet@ecp.fr

  COPYRIGHT:

    copyright (C) 2009  clement.courbet@ecp.fr

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

  CHANGE HISTORY:

    4 December 2009 - created

===============================================================================
*/
#ifndef SVREADER_HZIP_H
#define SVREADER_HZIP_H

#include "svreader.h"
#include "hzip.h"

#include <vector>
#include <stdio.h>

//undefine this to output a file for each vertex property instead of one file for all.
//defining it does not work...
//#define MULTIPLEX

class SVreader_hzip : public SVreader
{
	public:

		// SVreader interface function implementations

		SVevent read_element();
		SVevent read_event();

		void close ( bool close_file=true );

		// SVreader_sva functions

		bool open ( FILE* file );

		SVreader_hzip();
		~SVreader_hzip();

	private:
		FILE* file;
#ifndef MULTIPLEX
		FILE *conn_file;
		FILE **geo_file;
#endif
		HZMstream *conn_stream;
		HZNstream **geo_stream; //the HZIP streams
		I32 next_idx[8]; //holds the indices of the next cell
		bool decoded[8]; //holds the decoded state for each vertex in the next cell
		float *next_props; //holds the properties of the vertices of the next cell, in the order v0[0], v1[0], v2[0], ..., v0[1], v1[1], ...
		bool hasNextCell;
		std::vector<bool> *vertexDecoded; //tells if vertex index i has been decoded
		float *vertices; //vertex properties

		bool readNextCell();
};

#endif
