/*
===============================================================================

  FILE:  SVreader_svh.h
  
  CONTENTS:
  
    Reads a Streaming Volume Hex Mesh from a compressed format.
    
  PROGRAMMERS:
  
    clement.courbet@ecp.fr
  
  COPYRIGHT:
  
    copyright (C) 2009  clement courbet
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    05 Octob 2009 -- adapted from svreader_svc.h

===============================================================================
*/
#ifndef SVREADER_SVH_H
#define SVREADER_SVH_H

#include "svreader.h"

#include <stdio.h>

class SVreader_svh : public SVreader
{
public:

  // additional mesh variables
  int nbits;

  // SVreader interface function implementations

  SVevent read_element();
  SVevent read_event();

  void close(bool close_file=true);

  // SVreader_svh functions

  bool open(FILE* file);

  SVreader_svh();
  ~SVreader_svh();

private:
  FILE* file;
  I32 have_new, next_new, *new_vertices;
  int have_hexahedron;
  I32 have_finalized, next_finalized, *finalized_vertices;
  int cellDataBits;

  void read_header();
  int decompress_hexahedron();

  void createAddAndDecompressV0(void *vertices);
  void createAddAndDecompressV1(void *vertices);
  void createAddAndDecompressV2(void *vertices);
  void createAddAndDecompressV3(void *vertices);
  void createAddAndDecompressV4(void *vertices);
  void createAddAndDecompressV7(void *vertices);
  void createAddAndDecompressV5(void *vertices);
  void createAddAndDecompressV6(void *vertices);
};

#endif
