/*
===============================================================================

  FILE:  SVreadPreAsCompactPre.h
  
  CONTENTS:
  
    Reads a *pre-order* Streaming Volume mesh as a *compact* *pre-order* mesh
    with immediate finalization. This reorders and reindexes of the vertices.
    The 'preserve_order' flag specifies whether the algorithm is allowed to
    reorder cells.

    If the algorithm must preserve the cell order then the (cell-)span of the
    layout dictates the maximal number of cells it needs to buffer. This number
    will be prohibitively large for meshes that were created with some kind of
    depth first traversal. If the algorithm is allowed to reorder cells, then
    the (vertex-)width of the stream roughly dictates the maximal number of
    cells it needs to buffer.
  
  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2005  martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    21 February 2005 -- adapted from the similar streaming mesh version
  
===============================================================================
*/
#ifndef SVREAD_PRE_AS_COMPACT_PRE_H
#define SVREAD_PRE_AS_COMPACT_PRE_H

#include "svreaddereferenced.h"

class SVreadPreAsCompactPre : public SVreadDereferenced
{
public:

  // SVreader interface function implementations

  SVevent read_element();
  SVevent read_event();

  void close(bool close_file=true);

  // SVreadPreAsCompactPre functions

  bool open(SVreader* svreader, bool preserve_order=true);

  SVreadPreAsCompactPre();
  ~SVreadPreAsCompactPre();

private:
  SVreader* svreader;
  bool preserve_order;

  int have_new, next_new;
  int new_vertices[8];
  int have_cell;
  int have_finalized, next_finalized;
  int finalized_vertices[8];

  int read_preorder_input();
  int read_cell();
};

#endif
