/*
===============================================================================

  FILE:  SVwriteBuffered.h
  
  CONTENTS:
  
    Writes a Streaming Volume Mesh with a "little-cache" aware greedy reordering
    of hexahedra that is subject to a constraint of maximal delay of hexahedra.

  PROGRAMMERS:
  
    clement.courbet@ecp.fr
  
  COPYRIGHT:
  
    copyright (C) 2009  clement courbet
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
        15 October 2009 -- Adapted from <svwritebuffered> to handle hexahedrons
  
===============================================================================
*/
#ifndef SVWRITE_BUFFERED_HEX_H
#define SVWRITE_BUFFERED_HEX_H

//choose one of these:
//#define PICK_OLDEST_WITH_FACE_IN_CACHE
//#define PICK_OLDEST_WITH_MOST_FACES_IN_EXTENDED_CACHE
//#define PICK_MOST_VISITED_VERTICES_WITH_FACE_IN_CACHE
//#define PICK_BEST_OP_WITH_FACE_IN_CACHE
#define PICK_MOST_SPIRALLING

#ifdef PICK_BEST_OP_WITH_FACE_IN_CACHE
	#include "svwriter_svh.h"
#else
	#include "svwriter.h"
#endif

class SVwriteBufferedHex : public SVwriter
{
public:
  // svwriter interface function implementations

  void add_comment(const char* comment);

  void set_nverts(I32 nverts);
  void set_ncells(I32 ncells);
  void set_v_pnum_f(I32 v_pnum_f);
  void set_v_pbox_f(const float* v_pmin_f, const float* v_pmax_f);
  void set_c_pnum_f(I32 c_pnum_f);
  void set_c_pbox_f(const float* c_pmin_f, const float* c_pmax_f);

  void write_vertex(const float* v_data_f);

  void write_tetrahedron(const I32* c_idx, const bool* c_final=0);
  void write_tetrahedron(const I32* c_idx, const float* c_data_f, const bool* c_final=0);

  void write_hexahedron(const I32* c_idx, const bool* c_final=0);
  void write_hexahedron(const I32* c_idx, const float* c_data_f, const bool* c_final=0);

  void write_finalized(I32 final_idx);

  void close(bool close_file=true, bool update_buffer=false);

  // svwritebuffered functions

#ifdef PICK_BEST_OP_WITH_FACE_IN_CACHE
  bool open(SVwriter_svh* svwriter, int max_delay=100);
#else
  bool open(SVwriter* svwriter, int max_delay=100);
#endif

  SVwriteBufferedHex();
  ~SVwriteBufferedHex();

private:
#ifdef PICK_BEST_OP_WITH_FACE_IN_CACHE
  SVwriter_svh* svwriter;
#else
  SVwriter* svwriter;
#endif
  int max_delay;

  void write_hexahedron_delayed();

#ifdef PICK_BEST_OP_WITH_FACE_IN_CACHE
  void* find_hexahedron ( int dirty );
#endif
};

#endif
