/*
===============================================================================

  FILE:  SVwriter_svb.h
  
  CONTENTS:
  
    Writes a Streaming Volume Mesh in an efficient binary format. Currently
    this format only supports meshes with three coordinates per vertex plus
    one scalar value (that may be either a float or an int).

  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2004  martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    27 May 2005 -- inital version (three coordinates + scalar value per vertex)
    23 May 2005 -- created the day after Henna's swollen Ultimate ankle
  
===============================================================================
*/
#ifndef SVWRITER_SVB_H
#define SVWRITER_SVB_H

#include "svwriter.h"

#include <stdio.h>

class SVwriter_svb : public SVwriter
{
public:

  // SVwriter interface function implementations

  void write_vertex(const float* v_prop_f);

  void write_tetrahedron(const I32* c_idx, const bool* c_final=0);
  void write_tetrahedron(const I32* c_idx, const float* c_prop_f, const bool* c_final=0);

  void write_hexahedron(const I32* c_idx, const bool* c_final=0);
  void write_hexahedron(const I32* c_idx, const float* c_data_f, const bool* c_final=0);

  void write_finalized(I32 final_idx);

  void close(bool close_file=true, bool update_header=true);

  // SVwriter_svb functions

  void set_endianness(bool big_endian);

  bool open(FILE* file);

  SVwriter_svb();
  ~SVwriter_svb();

private:
  FILE* file;

  void write_header();
  void write_buffer();
  void write_buffer_remaining();
  void update_header();

  bool endian_swap;
  int update_seek_pos;

  I32 element_number;
  U32 element_descriptor;
  I32* element_buffer;
};

#endif
