/*
===============================================================================

  FILE:  SVwriter_svc.h
  
  CONTENTS:
  
    Writes a Streaming Volume Mesh in a compressed format.
    
  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2005  martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    30 April 2005 -- now also using an edge cache
    28 April 2005 -- completed version v1 which also uses a face cache
    22 April 2005 -- completed version v0 which only uses a vertex cache
    23 Februar 2005 -- created after a gigantic ALL-YOU-CAN-EAT sushi dinner

===============================================================================
*/
#ifndef SVWRITER_SVC_H
#define SVWRITER_SVC_H

#include "svwriter.h"

#include <stdio.h>

class SVwriter_svc : public SVwriter
{
public:

  // additional mesh variables
  int nbits;

  // SVwriter interface function implementations

  void set_v_pnum_f(I32 v_pnum_f);

  void write_vertex(const float* v_prop_f);

  void write_tetrahedron(const I32* c_idx, const bool* c_final=0);
  void write_tetrahedron(const I32* c_idx, const float* c_prop_f, const bool* c_final=0);

  void write_hexahedron(const I32* c_idx, const bool* c_final=0);
  void write_hexahedron(const I32* c_idx, const float* c_data_f, const bool* c_final=0);

  void write_finalized(I32 final_idx);

  void close(bool close_file=true, bool update_header=true);

  // SVwriter_svc functions

  bool open(FILE* file, int nbits=16, bool cbm_precision=false, int high_bits=8);

  SVwriter_svc();
  ~SVwriter_svc();

private:
  FILE* file;
  void write_header();
  void compress_tetrahedron(const I32* c_idx, const float* c_prop_f, const bool* c_final);
  bool cbm_precision;
  int high_bits;
};

#endif
