/*
===============================================================================

  FILE:  SVwriter_svc.h

  CONTENTS:

    Writes a Streaming Volume Hex Mesh in a compressed format.

  PROGRAMMERS:

    clement.courbet@ecp.fr

  COPYRIGHT:

    copyright (C) 2009  clement courbet

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

  CHANGE HISTORY:

    05 October 2009 -- modified from svc writer

===============================================================================
*/
#ifndef SVWRITER_SVH_H
#define SVWRITER_SVH_H

#include "svwriter.h"
	
#include <stdio.h>

class SVwriter_svh : public SVwriter
{
  public:

    // additional mesh variables
    int nbits;

    // SVwriter interface function implementations

    void set_v_pnum_f ( I32 v_pnum_f );

    void write_vertex ( const float* v_prop_f );

    void write_hexahedron ( const I32* c_idx, const bool* c_final=0 );
    void write_hexahedron ( const I32* c_idx, const float* c_prop_f, const bool* c_final=0 );

    void write_tetrahedron ( const I32* c_idx, const bool* c_final=0 ) {};
    void write_tetrahedron ( const I32* c_idx, const float* c_prop_f, const bool* c_final=0 ) {};

    void write_finalized ( I32 final_idx );

    void close ( bool close_file=true, bool update_header=true );

    // SVwriter_svc functions

    bool open ( FILE* file, int nbits=16, int high_bits=8, int nCellDataBits=0, int nCellDataHighBits=8 );

    SVwriter_svh();
    ~SVwriter_svh();

    //to help the delayed writer make its decisions: returns the operation that would be result in writing this hexahedron
    int test_write_hexahedron ( const I32* c_idx );

  private:
    FILE* file;
    void write_header();
    void compress_hexahedron ( const I32* c_idx, const float* c_prop_f, const bool* c_final );
    int high_bits;
    int cellDataHighBits;
    int cellDataBits;
};

#endif
